from enum import Enum


class Language(str, Enum):
    EN = "en"
    ZH = "zh"
    FR = "fr"
    DE = "de"
    IT = "it"
    JA = "ja"
    KO = "ko"
    PT = "pt"
    RU = "ru"
    ES = "es"
    NONE = "none"